@@ -2,6 +2,7 @@
 import sys, stat, time, math
 from bup import hashsplit, git, options, index, client
 from bup.helpers import *
+from bup.hashsplit import GIT_MODE_TREE, GIT_MODE_FILE, GIT_MODE_SYMLINK
 
 
 optspec = """
@@ -30,7 +31,7 @@ if not (opt.tree or opt.commit or opt.name):
 if not extra:
     o.fatal("no filenames given")
 
-opt.progress = (istty and not opt.quiet)
+opt.progress = (istty2 and not opt.quiet)
 opt.smaller = parse_num(opt.smaller or 0)
 if opt.bwlimit:
     client.bwlimit = parse_num(opt.bwlimit)
@@ -99,17 +100,17 @@ def _pop(force_tree):
     shalist = shalists.pop()
     tree = force_tree or w.new_tree(shalist)
     if shalists:
-        shalists[-1].append(('40000',
-                             git.mangle_name(part, 040000, 40000),
+        shalists[-1].append((GIT_MODE_TREE,
+                             git.mangle_name(part,
+                                             GIT_MODE_TREE, GIT_MODE_TREE),
                              tree))
     else:  # this was the toplevel, so put it back for sanity
         shalists.append(shalist)
     return tree
 
 lastremain = None
-lastprint = 0
 def progress_report(n):
-    global count, subcount, lastremain, lastprint
+    global count, subcount, lastremain
     subcount += n
     cc = count + subcount
     pct = total and (cc*100.0/total) or 0
@@ -141,17 +142,9 @@ def progress_report(n):
             remainstr = '%dm%d' % (mins, secs)
         else:
             remainstr = '%ds' % secs
-    if now - lastprint > 0.1:
-        progress('Saving: %.2f%% (%d/%dk, %d/%d files) %s %s\r'
-                 % (pct, cc/1024, total/1024, fcount, ftotal,
-                    remainstr, kpsstr))
-        lastprint = now
-
-
-def vlog(s):
-    global lastprint
-    lastprint = 0
-    log(s)
+    qprogress('Saving: %.2f%% (%d/%dk, %d/%d files) %s %s\r'
+              % (pct, cc/1024, total/1024, fcount, ftotal,
+                 remainstr, kpsstr))
 
 
 indexfile = opt.indexfile or git.repo('bupindex')
@@ -170,7 +163,7 @@ total = ftotal = 0
 if opt.progress:
     for (transname,ent) in r.filter(extra, wantrecurse=wantrecurse_pre):
         if not (ftotal % 10024):
-            progress('Reading index: %d\r' % ftotal)
+            qprogress('Reading index: %d\r' % ftotal)
         exists = ent.exists()
         hashvalid = already_saved(ent)
         ent.set_sha_missing(not hashvalid)
@@ -202,10 +195,10 @@ for (transname,ent) in r.filter(extra, wantrecurse=wantrecurse_during):
         else:
             status = ' '
         if opt.verbose >= 2:
-            vlog('%s %-70s\n' % (status, ent.name))
+            log('%s %-70s\n' % (status, ent.name))
         elif not stat.S_ISDIR(ent.mode) and lastdir != dir:
             if not lastdir.startswith(dir):
-                vlog('%s %-70s\n' % (status, os.path.join(dir, '')))
+                log('%s %-70s\n' % (status, os.path.join(dir, '')))
             lastdir = dir
 
     if opt.progress:
@@ -246,7 +239,7 @@ for (transname,ent) in r.filter(extra, wantrecurse=wantrecurse_during):
             if lastskip_name and lastskip_name.startswith(ent.name):
                 ent.invalidate()
             else:
-                ent.validate(040000, newtree)
+                ent.validate(GIT_MODE_TREE, newtree)
             ent.repack()
         if exists and wasmissing:
             count += oldsize
@@ -255,9 +248,8 @@ for (transname,ent) in r.filter(extra, wantrecurse=wantrecurse_during):
     # it's not a directory
     id = None
     if hashvalid:
-        mode = '%o' % ent.gitmode
         id = ent.sha
-        shalists[-1].append((mode, 
+        shalists[-1].append((ent.gitmode, 
                              git.mangle_name(file, ent.mode, ent.gitmode),
                              id))
     else:
@@ -272,7 +264,8 @@ for (transname,ent) in r.filter(extra, wantrecurse=wantrecurse_during):
                 lastskip_name = ent.name
             else:
                 try:
-                    (mode, id) = hashsplit.split_to_blob_or_tree(w, [f],
+                    (mode, id) = hashsplit.split_to_blob_or_tree(
+                                            w.new_blob, w.new_tree, [f],
                                             keep_boundaries=False)
                 except IOError, e:
                     add_error('%s: %s' % (ent.name, e))
@@ -290,12 +283,12 @@ for (transname,ent) in r.filter(extra, wantrecurse=wantrecurse_during):
                     add_error(e)
                     lastskip_name = ent.name
                 else:
-                    (mode, id) = ('120000', w.new_blob(rl))
+                    (mode, id) = (GIT_MODE_SYMLINK, w.new_blob(rl))
             else:
                 add_error(Exception('skipping special file "%s"' % ent.name))
                 lastskip_name = ent.name
         if id:
-            ent.validate(int(mode, 8), id)
+            ent.validate(mode, id)
             ent.repack()
             shalists[-1].append((mode,
                                  git.mangle_name(file, ent.mode, ent.gitmode),
