@@ -136,6 +136,7 @@ class Entry:
     def validate(self, gitmode, sha):
         assert(sha)
         assert(gitmode)
+        assert(gitmode+0 == gitmode)
         self.gitmode = gitmode
         self.sha = sha
         self.flags |= IX_HASHVALID|IX_EXISTS
@@ -454,7 +455,7 @@ def reduce_paths(paths):
 
 def merge(*iters):
     def pfunc(count, total):
-        progress('bup: merging indexes (%d/%d)\r' % (count, total))
+        qprogress('bup: merging indexes (%d/%d)\r' % (count, total))
     def pfinal(count, total):
-        log('bup: merging indexes (%d/%d), done.\n' % (count, total))
+        progress('bup: merging indexes (%d/%d), done.\n' % (count, total))
     return merge_iter(iters, 1024, pfunc, pfinal, key='name')
